----------------------------------------------------------------------------------
-- ********* Fleatiny-FPGA Platform top-level module ***********
-- This is basically a wrapper allows connection of user projects to 
-- Fleatiny-FPGA's on-board hardware
--
-- Creation Date: 4th August 2013
-- Author: Valentin Angelovski
--
-- 2013 - Valentin Angelovski
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;

entity FleaFPGA_2v4 is

  port(
    -- System clock and reset
	sys_clock		: in    std_logic;  -- main clock input from external clock source
	sys_reset		: in    std_logic;  -- main clock input from external RC reset circuit

    -- On-board user buttons and status LEDs
	n_pb1			: in    std_logic;  -- main clock input from external clock source
	n_pb2			: in    std_logic;  -- main clock input from external RC reset circuit
	n_led1			: buffer  std_logic;
	n_led2			: buffer  std_logic;
	n_led3			: buffer  std_logic;
	n_led4			: buffer  std_logic;

    -- USB Host (CH376T) interface
	host_reset		: buffer  std_logic;
	host_cs			: buffer  std_logic;
	host_sck		: buffer  std_logic;	
	host_spi		: buffer  std_logic;
	host_tx_o 		: out  std_logic;
	host_rx_i 		: in  std_logic;
	
    -- USB Slave (FT230x) interface 
	slave_tx_o 		: out  std_logic;
	slave_rx_i 		: in  std_logic;
	
    -- User GPIO (18 I/O pins) Header
	GPIO 			: inout std_logic_vector(18 downto 1);  -- GPIO Header pins available as one data block
	
	-- SDRAM interface (For use with 16Mx16bit or 32Mx16bit SDR DRAM, depending on version)
	Dram_Clk		: out   std_logic;     -- clock to SDRAM
	Dram_n_Ras		: out   std_logic;     -- SDRAM RAS
	Dram_n_Cas		: out   std_logic;     -- SDRAM CAS
	Dram_n_We		: out   std_logic;     -- SDRAM write-enable
	Dram_BA			: out   std_logic_vector(1 downto 0);   -- SDRAM bank-address
	Dram_Addr		: out   std_logic_vector(12 downto 0);  -- SDRAM address bus
	Dram_Data		: inout std_logic_vector(15 downto 0);  -- data bus to/from SDRAM
	Dram_n_cs		: out   std_logic; 
	--Dram_dqm		: out   std_logic_vector(1 downto 0); 
	Dram_DQMH		: out   std_logic;
	Dram_DQML		: out   std_logic;	
	
	-- VGA interface (Note: 10-bit color organized as RGB = 3/4/3-bits)
	vga_vs			: out   std_logic; 
	vga_hs			: out   std_logic; 
	vga_red			: out   unsigned(3 downto 0); 
	vga_green		: out   unsigned(3 downto 0); 
	vga_blue		: out   unsigned(3 downto 0); 
	
	-- SD/MMC Interface (Support either SPI or nibble-mode)
	mmc_dat1		: in   std_logic; 
	mmc_dat2		: in   std_logic; 
	mmc_n_cs		: out   std_logic; 
	mmc_clk			: out   std_logic; 
	mmc_mosi		: out   std_logic; 
	mmc_miso		: in   std_logic; 

	-- Audio out (stereo-PWM) interface	
	Audio_l			: out   std_logic; 
	Audio_r			: out   std_logic; 
	
	-- PS2 interface (Both ports accessible via Y-splitter cable)
	PS2_clk1		: inout    std_logic;
	PS2_data1		: inout    std_logic;
	PS2_clk2		: inout     std_logic;
	PS2_data2		: inout     std_logic

    );
end FleaFPGA_2v4;


architecture arch of FleaFPGA_2v4 is

	signal ps2m_clk_in : std_logic;
	signal ps2m_clk_out : std_logic;
	signal ps2m_dat_in : std_logic;
	signal ps2m_dat_out : std_logic;

	signal ps2k_clk_in : std_logic;
	signal ps2k_clk_out : std_logic;
	signal ps2k_dat_in : std_logic;
	signal ps2k_dat_out : std_logic;

  signal clk125             :std_logic;
  signal cpu_clk             :std_logic;
  signal xmit				:std_logic;
  
  signal h_sync 			:std_logic;
  signal v_sync 			:std_logic;
		
  signal v_Red :unsigned(7 downto 0);
  signal v_Green :unsigned(7 downto 0);
  signal v_Blue :unsigned(7 downto 0);
  
begin
  -- Housekeeping logic for unwanted peripherals on FleaFPGA board goes here!!!
  -- (Note: comment out any of the following code lines if peripheral is required)

--n_led1 <= '0';
--n_led2 <= '0';
--n_led3 <= '0';
--n_led4 <= '0';
Dram_Addr(12) <= '0';
--Dram_Clk <= sys_clock;


	ps2m_dat_in<=PS2_data1;
	PS2_data1 <= '0' when ps2m_dat_out='0' else 'Z';
	ps2m_clk_in<=PS2_clk1;
	PS2_clk1 <= '0' when ps2m_clk_out='0' else 'Z';

	ps2k_dat_in<=PS2_data2;
	PS2_data2 <= '0' when ps2k_dat_out='0' else 'Z';
	ps2k_clk_in<=PS2_clk2;
	PS2_clk2 <= '0' when ps2k_clk_out='0' else 'Z';


  -- User HDL project code and port mappings goes in here!!!
  -- (Note: This example uses tobiflex's M68K-based System-on-chip)
  
 u0 : entity work.zpu_PLL
   port map(CLKI => sys_clock, CLKOP => clk125, CLKOS => Dram_Clk, CLKOS2 =>cpu_clk);
 

		

ZPU_SoC : entity work.TG68Test
 
	port map( 
	vga_red(7 downto 4) => vga_red,
	vga_green(7 downto 4)  => vga_green,
	vga_blue(7 downto 4)  => vga_blue,
		
	vga_vsync => vga_vs,
	vga_hsync => vga_hs,
	
	-- System signals
	clk => clk125,
	--clk => sys_clock,
	reset_in => n_pb2,
	
	-- SDRAM	
	sdr_data => Dram_Data,
	sdr_addr => Dram_Addr(11 downto 0),
	sdr_dqm(0)  => Dram_DQML,
	sdr_dqm(1)  => Dram_DQMH,	
	sdr_we 	 => Dram_n_We,
	sdr_cas  => Dram_n_Cas,
	sdr_ras  => Dram_n_Ras,
	sdr_cs	 => Dram_n_cs,
	sdr_ba	 => Dram_BA,
		
	-- SPI signals	
	sd_miso => mmc_miso,
	sd_mosi => mmc_mosi,
	sd_clk	 => mmc_clk,
	sd_cs 	 => mmc_n_cs,
	
	-- UART
	rxd	=> slave_rx_i,
	txd	=> slave_tx_o,
	
	-- PS/2
	ps2k_clk_in => ps2k_clk_in,
	ps2k_dat_in => ps2k_dat_in,
	ps2k_clk_out => ps2k_clk_out,
	ps2k_dat_out => ps2k_dat_out,
	ps2m_clk_in => ps2m_clk_in,
	ps2m_dat_in => ps2m_dat_in,
	ps2m_clk_out => ps2m_clk_out,
	ps2m_dat_out => ps2m_dat_out,
	
	switches => (others =>'0'),

	-- Misc
	--pausecpu => '1',
	--pausevga => '1',
	buttons => "111",
	LED_x4(0) => n_led1,
	LED_x4(1) => n_led2,
	LED_x4(2) => n_led3,
	LED_x4(3) => n_led4, 

	audio_pwm_r => Audio_r,
	audio_pwm_l => Audio_l	
	
	);

end architecture;

